import { toHtml } from "hast-util-to-html";
import { getHighlighter } from "shiki";
import { parseMetadata } from "../parse-metadata/index.js";
import { renderToHast } from "./render-to-hast.js";
import { escapeHtml } from "./utils.js";
/**
 * Create MDsveX code highlighting function.
 *
 * @param options - Shiki highlighter options object.
 * @returns A Promise of a MDsvex highlighter function.
 * @public
 */
export async function createHighlighter(options) {
    const shikiHighlighter = await getHighlighter(options);
    /**
     * Highlighter function for Mdsvex codeblocks.
     *
     * [reference](https://github.com/atomiks/rehype-pretty-code/blob/e24f8415c8264dc868006ced839d3cb7445e716a/src/index.js#L50)
     * @param code - Raw source code to highlight.
     * @param lang - Programming language.
     * @param metastring - Code block fence metadata.
     * @returns HTML string.
     */
    return function highlighter(code, lang, metastring) {
        if (!lang)
            return `<pre><code>${escapeHtml(code)}</code></pre>`;
        const tokens = shikiHighlighter.codeToThemedTokens(code, lang || "txt");
        const metadata = parseMetadata(metastring);
        const tree = renderToHast({ tokens, metadata });
        /**
         * {@link hast-util-to-html#toHtml | toHtml} escapes `&` and `<`, so `escapeHtml` has to be called here.
         */
        return escapeHtml(toHtml(tree));
    };
}
//# sourceMappingURL=highlighter.js.map